/*
 *  ax25spyd - An AX.25 monitorspy daemon
 *
 *  Copyright (C) 1999 Free Software Foundation, Inc.
 *  Copyright (C) 1999 Walter Koch, dg9ep
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: spy.c,v 1.3 1999/05/23 16:27:44 walter Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <syslog.h>

#include "monixd.h"



/*------------- Client & Spy ------------------------------------------------*/

void
foundSpyQSO( int iClient, struct t_QSOMheard* pQSO)
/* open a new spy for iClient on pQSO */
{
    if( pQSO == NULL )
	return;
    if( pQSO->spyclient[iClient] )
	return; /* already enabled */
    if( iClient > MAXCLIENTPERQSO ) { /* too many clients! */
	syslog(LOG_ERR, "cannot create spy, too many clients... %d",iClient);
	return;
    }
    pQSO->spyclient[iClient] = 1;
    if( fVerbose )
	syslog(LOG_INFO, "created spy on qso %d for client %d",pQSO->qsoid,iClient);
    /* give the clinet some feedback */
    lwrite( iClient, T_SPYHEADER, pQSO, sizeof(*pQSO) );
}


int
startSpyNewClientQsoId( int iClient, int id )
/* Start spy on qsoid id for client iClient */
{
    struct t_QSOMheard* p;

    p = searchQSOFromQSOid( id );
    if( p == NULL )
	return 0; /* not found */
    foundSpyQSO(iClient, p);
    if( p->pOtherqso && client[iClient].fAutoSpyOtherID )
	foundSpyQSO(iClient, p->pOtherqso);
    return 1;
}

int
stopSpyClientQsoId( int iClient, int id )
/* Stop spy on qsoid id for client iClient */
{
    struct t_QSOMheard* p;

    p = searchQSOFromQSOid( id );
    if( p == NULL )
	return 0; /* not found */
    p->spyclient[iClient] = 0;
    lwrite( iClient, T_SPYEND, p, sizeof(*p) );
    if( fVerbose )
	syslog(LOG_INFO, "closed spy for qso %d for client %d",p->qsoid,iClient);
    return 1;
}


void
startSpyNewQSO( struct t_QSOMheard* pNewQSO )
/* Will be called, if new or till now unknown QSO is detected */
{
    int i;
    /* search all Clients wheather they want to spy the new QSO pNewQSO
     * (filterexpression) */
    for(i=0;i<nClient;i++)
	 if( client[i].sSpyFilter )
	     if(    !memcmp(client[i].sSpyFilter, pNewQSO->fmcall.sCall, strlen(client[i].sSpyFilter) )
		 || !memcmp(client[i].sSpyFilter, pNewQSO->tocall.sCall, strlen(client[i].sSpyFilter) ) ) {
		     foundSpyQSO(i, pNewQSO);
	     }

    /* Now look for clients, which want to be informed about all new qsos */
    for(i=0;i<nClient;i++)
	if( client[i].fOfferNewQSO )
	    if( (pNewQSO->otherqsoid==0) || (client[i].fOfferAllNewQSO) ) {
	       /* offer only if the otherqso has not already be offerd (this
		* could have been disabled) */
		lwrite( i, T_QSOOFFER, pNewQSO, sizeof(*pNewQSO) );
		if( fVerbose )
		    syslog(LOG_INFO, "offer qso %d to client %d",pNewQSO->qsoid,i);
	    }

	/* If the otherqso is already spyed, then pNewQSO must bes spyed too */
    if( pNewQSO->pOtherqso )
	for(i=0;i<nClient;i++)
	    if( client[i].fAutoSpyOtherID ) {
		if( pNewQSO->pOtherqso->spyclient[i] ) {
		    foundSpyQSO(i, pNewQSO);
		}
    }
}


void
startSpyNewClient( int iClient, char *sFilter )
/* search all QSO wheather they want to be spied by new client */
{
    struct t_QSOMheard* p;
    int id;

    if( (id = atoi(sFilter)) > 0 ) { /* qsoid issued */
	if( startSpyNewClientQsoId(iClient, id) )
	    return; /* found */
    }

    for( p = pQSOMheardRoot; p != NULL; p = p->pNext )
	if(    !memcmp(sFilter, p->fmcall.sCall, strlen(sFilter) )
	    || !memcmp(sFilter, p->tocall.sCall, strlen(sFilter) ) ) {
		foundSpyQSO(iClient, p);
	}
}


/*----------------------------------------------------------------------*/

static void
mailSpy( struct t_ax25packet* pax25, struct t_QSOMheard* pQSO)
{
    /* check for Mailheader */
}

static void
spyToFile( struct t_ax25packet* pax25, struct t_QSOMheard* pQSO)
/* write data of pax25 to file.
 * pax25 contains a packet in correct order
 * This is really a HACK */
{
    FILE *f;
    char filename[90];
    /* $TODO PERFORMANCE */
    sprintf(filename, AX25VARDIR"/ax25spyd/%s-%d->%s-%d",
		       pQSO->fmcall.sCall, pQSO->fmcall.ssid,
		       pQSO->tocall.sCall, pQSO->tocall.ssid
    );
    /* $TODO PERFORMANCE */
    if( (f = fopen(filename, "a")) == NULL) {
	printf("can't open %s",filename);
	return;
    }
    /* $TODO WE CHANGE THE ORIGINAL PACKET DATA HERE ! AARGGHH! */
    convertBufferToReadable( pax25->pdata,  pax25->datalength );
    fwrite( pax25->pdata, pax25->datalength, 1, f);
    fclose(f);
}

static void
doSpy( struct t_ax25packet* pax25, struct t_QSOMheard* pQSO)
/* if this QSO is spyed, write Data to the client.
 * *pax25  belongs to  *pQSO
 */
{
    if( pax25->datalength <= 0 ) {
	return;
    }

    if( pax25->frametype != UI ) { /* UIs do not have numbers... */
       /* And here is the genius code to avoid dupes :-/ */
       if( pQSO->lastSpyedNumber == pax25->ns )
	   return; /* got it already */
       pQSO->lastSpyedNumber = pax25->ns;
    }

    lwriteSpy( pQSO, pax25->pdata, pax25->datalength );

    if( fDoMailSpy )
	mailSpy( pax25, pQSO );
    if( fSpyAllToFile )
	spyToFile( pax25, pQSO );
}

/*---------------- Dispatcher ---------------------------------------*/
void
tryspy( struct t_ax25packet* pax25 )
/* called by ax25_dump (ax25dump.c) */
{
    if( !pax25->valid )
	return;
    /* update mheard data */
    doCallMheard(pax25);
    /* look for matching QSO */
    if( pax25->pQSO == NULL )
	return;
    /* End of QSO? */
    if( pax25->frametype == DISC ) {
	delQSO( pax25->pQSO );
	pax25->pQSO = NULL;
	return;
    }

    tryspydxcluster(pax25);
    /*	  if( (pax25->pid == PID_TEXT) && (pax25->frametype == UI) ) {
     *	      tryspyHeader(pax25,NULL,"MAIL");
     *	      tryspyHeader(pax25,NULL,"MAILS");
     *	  }
     */
    doSpy(pax25,pax25->pQSO);
}


